#!/bin/zsh

# mapWOC
#
# Description: Automatically create sha1 check sums for all files of
#              the host system.  Automatically run at boottime of the
#              grml live system.
#
# Author(s):
# Sascha Wilde <wilde@intevation.de>
#
# Copyright:
# Copyright (C) 2011 Bundesamt für Sicherheit in der Informationstechnik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2,
# or, at your option, any later version as published by the Free
# Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

trap "" INT

typeset -A FSTAB

# Real production FSTAB:
FSTAB=(
  /dev/cciss/c0d0p1  /
  /dev/vg0/usr       /usr
  /dev/vg0/var       /var
  /dev/vg0/tmp       /tmp
  /dev/vg0/home      /home
  /dev/vg0/images    /usr/share/mapwoc/images
)
MNTORDER=(
  /
  /usr
  /var
  /tmp
  /home
  /usr/share/mapwoc/images
)

DIRNAME_EXTRA=''
DIRNAME=''
CHKSUMDIR='CHECKSUMS'
BASEDIR="/live/image/$CHKSUMDIR"
ERRFILE='/tmp/FATALERROR'
HOSTNAME=''
MNT='/mnt/tmp0'

reboot_system() {
  umount_hostfs | dialog --title "Un-Mounting host file systems" --progressbox 20 75
  dialog --infobox "The System will be shutdown for reboot now!" 3 47
  sleep 2
  reboot
}


fatal() {
  dialog --title "FATAL ERROR!" --msgbox "A fatal error accured:
$1
This is probably indicating a serious problem, which should be
further investigated by an experienced Systemadministrator!
Will reboot now." 15 75
  reboot_system
}


get_filename_extra() {
  local NAMESTRING_OK=no
  exec 3>&1
  until [[ $NAMESTRING_OK == "yes" ]]
  do
    DIRNAME_EXTRA=`dialog --inputbox \
"You might enter a comment which will be appended to the filename
of the created check sums file:" 9 75 2>&1 1>&3`
    if grep -vq '[/\$ ]' <<<"$DIRNAME_EXTRA"
    then
      NAMESTRING_OK="yes"
    else
      dialog --title "Illegal input!" \
        --msgbox \
        'The comment string may not contain spaces or any of the following 
charcters: / \ $' \
        6 75 >&3
    fi
  done
  exec 3>&-
  [ "$DIRNAME_EXTRA" ] && DIRNAME_EXTRA="_$DIRNAME_EXTRA"
}


check_directory() {
  mount -o remount,rw /live/image || \
    fatal "Could not remount USB stick in writeable mode!"
  if [[ ! -d $BASEDIR ]] ; then
    if dialog --title "Missing check sums directory" \
      --yesno "The check sums directory $CHKSUMDIR is missing create it?" \
      6 75
    then
      mkdir $BASEDIR
    else
      fatal "Can't proceed without checkums directory!"
    fi
  fi
}


mount_hostfs() {
  /etc/init.d/lvm2 start 2>&1
  mkdir -p $MNT
  for mntpt in ${MNTORDER[*]} 
  do
    mount 2>&1 -v ${(k)FSTAB[(r)$mntpt]} ${MNT}$mntpt || \
      echo "could not mount $fs as ${MNT}${FSTAB[$fs]}" >$ERRFILE
  done
}


umount_hostfs() {
  for mntpt in ${(aO)MNTORDER[*]} 
  do
    umount 2>&1 ${MNT}${mntpt}
  done
}


setup() {
  dialog --yesno "Welcome to the mapwoc checksum creation system.

This programm creates SHA1 check sums for all files on the mapwoc
system and saves them to a file for later inspection.  

This will take a fair ammount of time!

Proceed?" 12 75 || reboot_system
  rm $ERRFILE  # in case we get manually started a second time
  get_filename_extra
  DIRNAME=`date "+%Y%m%dT%H%M%S${DIRNAME_EXTRA}"`
  check_directory
  mount_hostfs | dialog --title "Mounting host file systems" --progressbox 20 75
  if [[ -f $ERRFILE ]] ; then
    fatal "$(< $ERRFILE )"
  fi
  HOSTNAME=`cat $MNT/etc/hostname`
  if [[ -z "$HOSTNAME" ]] ; then
    fatal "Could not get hostname of the mapwoc system!"
  fi
}


copy_logbuch() {
  dialog --infobox "Copying logbuch.txt ..." 3 27
  cp $MNT/etc/logbuch.txt "$BASEDIR/$DIRNAME/" || \
    fatal "Could not copy logbuch!"
}


checksum_file() {
  echo "Creating sha1sum for ${1#./}..."
  sha1sum 2>$ERRFILE "$1" | sed 's|\./||' >>"$2"
}


create_checksums() {
  mkdir "$BASEDIR/$DIRNAME" || \
    fatal "Could not create checksums directory!"
  cd $MNT
  IFS=''
  find . -type f | while read filename
  do
    checksum_file "$filename" "$BASEDIR/$DIRNAME/$HOSTNAME.sha1sums"
  done | dialog --title "Creating Check Sums" --progressbox 20 75
  if [[ -f $ERRFILE && -n $(< $ERRFILE ) ]] ; then
    fatal "$(< $ERRFILE )"
  else
    rm $ERRFILE
  fi
}


create_meta_checksums() {
  cd $BASEDIR/$DIRNAME
  for filename in "$HOSTNAME.sha1sums" "logbuch.txt"
  do
    checksum_file "$filename" "$BASEDIR/$DIRNAME/$DIRNAME.sha1sums"
  done | dialog --title "Creating Check Sums" --progressbox 20 75
  if [[ -f $ERRFILE && -n $(< $ERRFILE ) ]] ; then
    fatal "$(< $ERRFILE )"
  else
    rm $ERRFILE
  fi
}


# --------------------------------------------------------------------
# main programm:

setup
create_checksums
copy_logbuch
create_meta_checksums
dialog --title "Task complete" --msgbox \
"The new checksums file can be found on this USB stick in 
$CHKSUMDIR/$DIRNAME" 6 75
reboot_system
