#!/bin/zsh

# mapWOC
#
# Description: Automatically reset the host system to factory delivery
#              status.  Automatically run at boottime of the grml live
#              system.
#
# Author(s):
# Sascha Wilde <wilde@intevation.de>
#
# Copyright:
# Copyright (C) 2011 Bundesamt für Sicherheit in der Informationstechnik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2,
# or, at your option, any later version as published by the Free
# Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

trap "" INT

typeset -A LVS DEVS UUID

TARSDIR='/live/image/TARS'
DATADIR='/live/image/DATA'
ERRFILE='/tmp/FATALERROR'
MNT='/mnt/tmp0'
DEVS=(
  DISK /dev/cciss/c0d0
  root /dev/cciss/c0d0p1
  swap /dev/cciss/c0d0p2
  pv   /dev/cciss/c0d0p3
)
LVS=(
  home     1G
  images 200G
  tmp     20G
  usr      2G
  var     10G
)
UUID=(
  root 67ffad22-5153-4a21-bb22-974faccd444d
  usr  30d1c319-dad3-4ee5-8c6e-32c816718234
)

reboot_system() {
  umount_hostfs | dialog --title "Un-Mounting host file systems" --progressbox 20 75
  dialog --infobox "The System will be shutdown for reboot now!" 3 47
  sleep 2
  reboot
  exit 23  # only executed when reboot fails!
}

fatal() {
  dialog --title "FATAL ERROR!" --msgbox "A fatal error accured:
$1
This is probably indicating a serious problem, which should be
further investigated by an experienced Systemadministrator!
Will reboot now." 15 75
  reboot_system
}

check_errfile() {
  if [[ -f $ERRFILE && -n $(< $ERRFILE ) ]] ; then
    fatal "$(< $ERRFILE )"
  else
    rm $ERRFILE
  fi
}

check_tars() {
  cd "$TARSDIR" || fatal "Missing archiv directory $TARSDIR"
  sha1sum 2>&1 2>>$ERRFILE -c SHA1SUMS | \
    dialog --title "Checking archive files" --progressbox 20 75
  check_errfile
}

check_prerequisites() {
  [ -b $DEVS[DISK] ] || fatal "Blockdevice $DEVS[DISK] missing!"
}

final_warning() {
  dialog --defaultno --yesno "Welcome to the mapwoc recovery system.

This programm will reset your mapWOC box to factory delivery status!

Be warned:  -- ALL HARDDISKS WILL BE REFORMATTED
            -- ALL PARTITIONS WILL BE RESET
            -- ALL YOUR DATA WILL BE LOST
            -- BASIC SYSTEM CONFIGURATION WILL BE NECESSARY

Really proceed?" 14 75 || reboot_system
}

restore_mbr_and_parttable() {
  ( echo "Restoring GRUB boot record:"
    dd 2>&1 if=${DATADIR}/mbr of=$DEVS[DISK] bs=446 count=1 || \
      echo >>$ERRFILE "Could not restore boot record to $DEVS[DISK]"
    echo "Restoring partition table::"
    sfdisk 2>&1 $DEVS[DISK] <${DATADIR}/parttable || \
      echo >>$ERRFILE "Could not restore partition table to $DEVS[DISK]"
  ) | \
    dialog --title "Restoring MBR and partition table" --progressbox 20 75
  check_errfile
}

create_lvs() {
  (
    # erase old lvm label
    dd 2>&1 if=/dev/zero of=$DEVS[pv] bs=512 count=2
    pvcreate 2>&1 2>>$ERRFILE $DEVS[pv]
    vgcreate 2>&1 2>>$ERRFILE vg0 $DEVS[pv]
    for name in ${(k)LVS} ; do
      lvcreate 2>&1 2>>$ERRFILE -n $name -L $LVS[$name] vg0
    done
  ) | \
    dialog --title "Creating logical volumes" --progressbox 20 75
  check_errfile
}

create_filesystems() {
  (
    mkfs.xfs 2>&1 2>>$ERRFILE $DEVS[root]
    xfs_admin 2>&1 2>>$ERRFILE -U $UUID[root] $DEVS[root]
    mkswap 2>&1 2>>$ERRFILE $DEVS[swap]
    for name in ${(k)LVS} ; do
      mkfs.xfs /dev/vg0/$name
      [ "$UUID[$name]" ] && xfs_admin 2>&1 2>$ERRFILE -U $UUID[$name] /dev/vg0/$name
    done
  ) | \
    dialog --title "Creating filesystems and swap" --progressbox 20 75
  check_errfile
}

extract_tars() {
  (
    mkdir -p $MNT
    mount 2>&1 2>>$ERRFILE $DEVS[root] $MNT && \
      tar 2>&1 2>>$ERRFILE -C $MNT -xvf $TARSDIR/root.tar.bz2 && \
      umount 2>&1 2>>$ERRFILE $MNT
    for name in ${(k)LVS} ; do
      if [[ -f $TARSDIR/$name.tar.bz2 ]] ; then
        echo "Restoring $name"
        mount 2>&1 2>>$ERRFILE /dev/vg0/$name $MNT && \
          tar 2>&1 2>>$ERRFILE -C $MNT -xvf $TARSDIR/$name.tar.bz2 && \
          umount 2>&1 2>>$ERRFILE $MNT
      fi
    done
  ) | \
    dialog --title "Extracting tar archives" --progressbox 20 75
  check_errfile
}

setup_grub() {
  # grub uses blocklists to get stage 1.5 on the mapWOC box, so we
  # must reinstall ist to get the correct blocks...
  (
    mount 2>&1 2>>$ERRFILE $DEVS[root] $MNT
    mount 2>&1 2>>$ERRFILE /dev/vg0/usr $MNT/usr
    $MNT/usr/sbin/grub-setup -d $MNT/boot/grub \
      -m $MNT/boot/grub/device.map -f $DEVS[DISK] || \
      echo >>$ERRFILE "Installing GRUB failed!"
    umount 2>&1 2>>$ERRFILE $MNT/usr
    umount 2>&1 2>>$ERRFILE $MNT
  ) | \
    dialog --title "Installing GRUB" --progressbox 20 75
  check_errfile
}


# --------------------------------------------------------------------
# main programm:

check_tars
check_prerequisites
final_warning
restore_mbr_and_parttable
create_lvs
create_filesystems
extract_tars
setup_grub
dialog --title "Task complete" --msgbox \
"The factory delivery status has been restored, will reboot now.

DON'T FORGET:
- You will have to fix up some system configuration manually (Network!)
- You will have to restore the images of the mapWOC virtual machines
  by copiing from your backup to /usr/share/mapwoc/images/

Good luck!" 12 75
reboot_system
