#!/bin/bash

# mapWOC
#
# Description: Install a minimal GRML live system together with the
#              custom mapWOC restore system on a USB-stick.
#
# Author(s):
# Sascha Wilde <wilde@intevation.de>
#
# Copyright:
# Copyright (C) 2007, 2008 Intevation GmbH
# Copyright (C) 2011 Bundesamt für Sicherheit in der Informationstechnik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2,
# or, at your option, any later version as published by the Free
# Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

VERBOSE=${VERBOSE:-1}
me=`basename $0`


usage() {
  cat <<EOF
USAGE: $me GRMLISO DEVICE

Install USB-drive live system to DEVICE (e.g. /dev/sdb1).
GRMLISO is the base GRML ISO image used.  It should point to 
"grml64-small_2010.12.iso".
EOF
  exit 0
}

verbose()
{
  [ $VERBOSE -eq 1 ] && echo "$1"
}

err() {
  echo >/dev/stderr "$me: ERR: $*"
  cleanexit 23
}

cleanexit() {
  mountpoint -q "$tmpmnt" && umount "$tmpmnt"
  [ -d "$tmpmnt" ] && rmdir "$tmpmnt"
  exit $1
}

# ----------------------------------------------------------------------

grmliso="$1"
dstdev="$2"

[ $# -eq 2 ]        || usage
[ -f "$grmliso" ]   || err "$grmliso not found."
[ -b "$dstdev" ]    || err "$dstdev is not a block device."
[ "`id -u`" -eq 0 ] || err "This script has to be run by root."
tmpmnt=`mktemp -d`  || err "Could not create temporary mountpoint."

verbose "Creating live system on $dstdev using $grmliso as base ISO."

verbose "Formating and labeling $dstdev ..."
mkfs.vfat -n GRMLCFG "$dstdev" \
  || err "Formating $dstdev failed!"

verbose "Installing GRML base system ..."
grml2usb --bootoptions="tz=Europe/Berlin noblank nodhcp" "$grmliso" "$dstdev" \
  || err "Installation of GRML live system failed!"

verbose "Mounting new system temporarily to $tmpmnt ..."
mount "$dstdev" "$tmpmnt" && \
  verbose "Copying grml.sh application script ..." && \
  cp grml.sh "$tmpmnt" && \
  verbose "Copying modified syslinux.cfg ..." && \
  cp syslinux.cfg "$tmpmnt/boot/syslinux/" && \
  verbose "Copying extra data ..." && \
  mkdir "$tmpmnt/DATA" && \
  cp mbr parttable "$tmpmnt/DATA/" && \
  verbose "Copying extra tars ..." && \
  mkdir "$tmpmnt/TARS" && \
  cp tars/* "$tmpmnt/TARS/" && \
  verbose "Copying modified syslinux.cfg ..." && \
  cp syslinux.cfg "$tmpmnt/boot/syslinux/" && \
  verbose "Unmounting ..." || err "Ouch! Could not copy stuff to new system..."

verbose "You new live system on $dstdev is now ready to boot!"

cleanexit 0
