# -*- coding: utf-8 -*-
# Description: Tests for the redirector module
#
# Copyright:
# Copyright (C) 2011 Bundesamt fur Sicherheit in der Informationstechnik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2,
# or, at your option, any later version as published by the Free
# Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

import unittest
import os
import os.path
import shutil
from mapwoc.lib.url import URL, create_URL_List
from mapwocgui.model.meta import Session
from mapwoc.redirector.redirector import *

#setup_logging(False, True)

URLLIST1 = [ "http://www.list-1-1.de",
             "http://www.list-1-2.de",
             "http://www.list-2-1.de" ]
URLLIST2 = [ "http://www.list-2-1.de",
             "http://www.list-2-2.de",
             "http://www.list-2-3.de",
             "http://www.list-2-4.de",
             "http://www.list-2-5.de" ]
URLLIST3 = [ "http://www.list-3-1.de",
             "http://www.list-3-2.de",
             "http://www.list-3-3.de",
             "http://www.list-3-4.de",
             "http://www.list-3-5.de" ]

class RedirectorMultiTest(unittest.TestCase):
    def setUp(self):
        urllist1, _invalids = create_URL_List("urllist1",
                                        "\n".join(URLLIST1))
        urllist2, _invalids = create_URL_List("urllist2",
                                        "\n".join(URLLIST2))
        urllist3, _invalids = create_URL_List("urllist2",
                                        "\n".join(URLLIST2))
        if not os.path.isdir("tests/tmp"):
            os.mkdir("tests/tmp")
        self.dbpath = "tests/tmp/tmpdb.db"
        shutil.copyfile("tests/dbs/database.db", self.dbpath)
        engine = create_engine('sqlite:///tests/tmp/tmpdb.db')
        __c = engine.connect()
        Session.configure(bind=engine)
        urlid = 2
        for urllist in [urllist1, urllist2, urllist3]:
            for url in urllist.urls:
                url.id=urlid
                urlid += 1
                Session.add(url)
                Session.commit()
            Session.add(urllist)
            Session.commit()
        Session.flush()
        self.redirector = Redirector(redirector_port=9991,
                                     ctrl_port=9992)

    def tearDown(self):
        Session.remove() # Necessary to "clear the cache"
        os.remove("tests/tmp/tmpdb.db")

    def assertNextUrl(self, url):
        "Asserts if next url does not equal url"
        self.assertEqual(self.redirector.get_next_url("the_test_hcsid", "0.0.0.0"),
                url)

    def testAll(self):
        """
        Combined test case. Not pretty but keeps us from having to start
        multiple redirectors
        """

        # Test get_next_url
        self.redirector.load_url_db("2,3,tests/tmp/tmpdb.db")
        combined_list = URLLIST1 + URLLIST2
        for u in combined_list:
            self.assertNextUrl(u)

        # Test looping
        for i in range(1,100):
            url = self.redirector.get_next_url("the_test_hcsid", "0.0.0.0")
            self.assertEqual(url == None, False)

        # Test state
        self.redirector.load_url_db("2,3,tests/tmp/tmpdb.db")
        state = self.redirector.get_state(number_next_urls=5)
        self.assertEqual(state["last_urls"], [])
        self.assertEqual(state["n_urls_served"], 0)
        for i in range(5):
            self.assertEqual(state["next_urls"][i], combined_list[i])
            self.assertNextUrl(combined_list[i])
        self.assertNextUrl(combined_list[5])
        self.assertNextUrl(combined_list[6])
        self.assertNextUrl(combined_list[7])
        state = self.redirector.get_state(number_next_urls=5)
        self.assertEqual(state["state"], "done")

        self.redirector.load_url_db("2,4,tests/tmp/tmpdb.db")

class TestMappingFromFile(unittest.TestCase):
    def setUp(self):
        invalidFormat = """
[empty]
[not empty]
http://www.valid.url
http://www.valid2.url
"""
        invalidFormat2 = """
[sadasd}
http://www.valid.url
http://www.valid2.url
"""


        invalidURLS = """
hattp://blub.de
http://valid.url
gubbelgubbel
"""

        validMapping = """
[test_machine]
http://gubbel.de


http://guppel.de
[test_machine2]

http://gubbel.de

http://guppel.de
"""
        if not os.path.isdir("tests/tmp"):
            os.mkdir("tests/tmp")
        with open(os.path.join("tests/tmp", "invalidFormat"),"w") as fname:
            fname.write(invalidFormat)
        with open(os.path.join("tests/tmp", "invalidFormat2"),"w") as fname:
            fname.write(invalidFormat2)
        with open(os.path.join("tests/tmp", "invalidURLS"),"w") as fname:
            fname.write(invalidURLS)
        with open(os.path.join("tests/tmp", "validMapping"),"w") as fname:
            fname.write(validMapping)

    def tearDown(self):
        for base in ('invalidFormat', 'invalidFormat2', 'invalidURLS',
                     'validMapping'):
            os.remove(os.path.join('tests/tmp', base))


    def test_invalidFormat(self):
        mapping = mapping_from_file(
                    os.path.join("tests/tmp/invalidFormat"))
        self.assertEqual(mapping.get("empty"), None,
                "Empty mapping returns not None: %s" % mapping)
        self.assertEqual(mapping["not empty"], [ "http://www.valid.url",
                                                  "http://www.valid2.url"],
                               "Mapping not empty is wrong: %s" % mapping)

    def test_invalidURLS(self):
        mapping = mapping_from_file(
              os.path.join("tests/tmp/invalidURLS"))
        self.assertEqual(mapping[""], [ "http://valid.url" ],
                "Invalid URLS not filtered: %s" % mapping)

    def test_validMapping(self):
        mapping = mapping_from_file(
              os.path.join("tests/tmp/validMapping"))
        self.assertEqual(mapping["test_machine"], ["http://gubbel.de",
                                                    "http://guppel.de", ],
                                                    mapping)
        self.assertEqual(mapping["test_machine2"], ["http://gubbel.de",
                                                     "http://guppel.de" ],
                                                    mapping)
